@@ -56,7 +56,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_wavpack_parse_debug);
 #define GST_CAT_DEFAULT gst_wavpack_parse_debug
 
 static inline GstWavpackParseIndexEntry *
-gst_wavpack_parse_index_entry_new ()
+gst_wavpack_parse_index_entry_new (void)
 {
   return g_slice_new (GstWavpackParseIndexEntry);
 }
@@ -147,7 +147,7 @@ gst_wavpack_parse_class_init (GstWavpackParseClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_wavpack_parse_finalize);
+  gobject_class->finalize = gst_wavpack_parse_finalize;
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_wavpack_parse_change_state);
 }
