@@ -1621,6 +1621,11 @@ gst_flv_demux_chain (GstPad * pad, GstBuffer * buffer)
     demux->offset = GST_BUFFER_OFFSET (buffer);
   }
 
+  if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)) {
+    GST_DEBUG_OBJECT (demux, "Discontinuity");
+    gst_adapter_clear (demux->adapter);
+  }
+
   gst_adapter_push (demux->adapter, buffer);
 
   if (demux->seeking) {
@@ -2404,13 +2409,31 @@ flv_demux_handle_seek_push (GstFlvDemux * demux, GstEvent * event)
 wrong_format:
   {
     GST_WARNING_OBJECT (demux, "we only support seeking in TIME format");
-    return gst_pad_push_event (demux->sinkpad, event);
+    gst_event_unref (event);
+    return FALSE;
   }
 }
 
 static gboolean
 gst_flv_demux_handle_seek_push (GstFlvDemux * demux, GstEvent * event)
 {
+  GstFormat format;
+
+  gst_event_parse_seek (event, NULL, &format, NULL, NULL, NULL, NULL, NULL);
+
+  if (format != GST_FORMAT_TIME) {
+    GST_WARNING_OBJECT (demux, "we only support seeking in TIME format");
+    gst_event_unref (event);
+    return FALSE;
+  }
+
+  /* First try upstream */
+  if (gst_pad_push_event (demux->sinkpad, gst_event_ref (event))) {
+    GST_DEBUG_OBJECT (demux, "Upstream successfully seeked");
+    gst_event_unref (event);
+    return TRUE;
+  }
+
   if (!demux->indexed) {
     guint64 seek_offset = 0;
     gboolean building_index;
@@ -2543,6 +2566,8 @@ gst_flv_demux_handle_seek_pull (GstFlvDemux * demux, GstEvent * event,
       demux->seek_event = gst_event_ref (event);
       demux->seek_time = seeksegment.last_stop;
       demux->state = FLV_STATE_SEEK;
+      /* do not know about succes yet, but we did care and handled it */
+      ret = TRUE;
       goto exit;
     }
     /* now index should be as reliable as it can be for current purpose */
@@ -2858,6 +2883,17 @@ gst_flv_demux_query (GstPad * pad, GstQuery * query)
       GstFormat fmt;
 
       gst_query_parse_seeking (query, &fmt, NULL, NULL, NULL);
+
+      /* First ask upstream */
+      if (fmt == GST_FORMAT_TIME && gst_pad_peer_query (demux->sinkpad, query)) {
+        gboolean seekable;
+
+        gst_query_parse_seeking (query, NULL, &seekable, NULL, NULL);
+        if (seekable) {
+          res = TRUE;
+          break;
+        }
+      }
       res = TRUE;
       if (fmt != GST_FORMAT_TIME || !demux->index) {
         gst_query_set_seeking (query, fmt, FALSE, -1, -1);
@@ -3074,7 +3110,7 @@ gst_flv_demux_class_init (GstFlvDemuxClass * klass)
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_flv_demux_dispose);
+  gobject_class->dispose = gst_flv_demux_dispose;
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_flv_demux_change_state);
