@@ -31,13 +31,6 @@
 GST_DEBUG_CATEGORY_STATIC (rtpgsmpay_debug);
 #define GST_CAT_DEFAULT (rtpgsmpay_debug)
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_gsm_pay_details =
-GST_ELEMENT_DETAILS ("RTP GSM payloader",
-    "Codec/Payloader/Network",
-    "Payload-encodes GSM audio into a RTP packet",
-    "Zeeshan Ali <zeenix@gmail.com>");
-
 static GstStaticPadTemplate gst_rtp_gsm_pay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -76,7 +69,10 @@ gst_rtp_gsm_pay_base_init (gpointer klass)
       gst_static_pad_template_get (&gst_rtp_gsm_pay_sink_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_gsm_pay_src_template));
-  gst_element_class_set_details (element_class, &gst_rtp_gsm_pay_details);
+  gst_element_class_set_details_simple (element_class, "RTP GSM payloader",
+      "Codec/Payloader/Network",
+      "Payload-encodes GSM audio into a RTP packet",
+      "Zeeshan Ali <zeenix@gmail.com>");
 }
 
 static void
